@@ -118,6 +118,12 @@ class ConstantExtendedHermiteProbabilistsFunction(Basis):
             DVr[:,i] = self.GradEvaluate(r, i, k, norm)
         return DVr
 
+    @staticmethod
+    def from_xml_element(node):
+        normalized = node.attrib.get('normalized', default=None)
+        if normalized is not None: normalized = bool(normalized)
+        return ConstantExtendedHermiteProbabilistsFunction(normalized=normalized)
+
 class HermiteProbabilistsRadialBasisFunction(Basis):
     r""" Construction of the Hermite Probabilists' Radial Basis Functions
 
@@ -126,30 +132,30 @@ class HermiteProbabilistsRadialBasisFunction(Basis):
 
     .. math::
 
-       \phi_i(x) = \begin{cases}
-       \exp\left( -\frac{(x-x_i)^2}{2\sigma^2_{i-1}} \right) & \text{if } x \leq x_i
-       \exp\left( -\frac{(x-x_i)^2}{2\sigma^2_{i}} \right) & \text{if } x > x_i
-       \end{cases}
+       \phi_i(x) = \exp\left( -\frac{(x-x_i)^2}{2\sigma^2_{i}} \right) 
 
-    where :math:`\sigma_i=x_{i+1} - x_{i}`, :math:`\sigma_0=\sigma_1` and
+    where :math:`\sigma_i=(x_{i+1} - x_{i-1})/2.`, :math:`\sigma_0=\sigma_1` and
     :math:`\sigma_N=\sigma_{N-1}`
 
     Args:
-      nknots (int): number of knots points :math:`x_i`
+      nbasis (int): number of knots points :math:`x_i`
       scale (float): scaling for the badwidth :math:`\sigma`.
     """
 
-    def __init__(self, nknots, scale=1.):
-        if nknots < 1:
-            raise ValueError("Range error. nknots >= 1 must hold")
-        self.nknots = nknots
+    def __init__(self, nbasis, scale=1.):
+        if nbasis < 0:
+            raise ValueError("Range error. nbasis >= 0 must hold")
+        self.nknots = nbasis+1
         self.hp = HermiteProbabilistsPolynomial()
         self.xknots, self.wknots = self.hp.GaussQuadrature(self.nknots-1)
-        self.sigma = np.zeros(self.nknots+1)
+        self.sigma = np.zeros(self.nknots)
         if self.nknots == 1:
             self.sigma[:] = 1.
+        elif self.nknots == 2:
+            self.sigma[0] = self.xknots[1] - self.xknots[0]
+            self.sigma[1] = self.sigma[0]
         else:
-            self.sigma[1:-1] = np.diff(self.xknots)
+            self.sigma[1:-1] = (self.xknots[2:] - self.xknots[:-2])/2.
             self.sigma[0] = self.sigma[1]
             self.sigma[-1] = self.sigma[-2]
         self.sigma *= scale
@@ -178,15 +184,10 @@ class HermiteProbabilistsRadialBasisFunction(Basis):
         """
         if N > self.nknots:
             raise ValueError("N must be <= than the number of knots")
-        leq = (x <= self.xknots[N])
-        left = np.where(leq)[0]
-        right = np.where(np.logical_not(leq))[0]
-        out = np.zeros(x.shape[0])
         diff = x - self.xknots[N]
-        out[left] = np.exp( - (diff[left])**2./(2.*self.sigma[N-1]**2.) )
-        out[right] = np.exp( - (diff[right])**2./(2.*self.sigma[N]**2.) )
+        out = np.exp( - diff**2./(2.*self.sigma[N]**2.) )
         if extended_output:
-            return (out, left, right, diff)
+            return (out, diff)
         else:
             return out
 
@@ -195,18 +196,15 @@ class HermiteProbabilistsRadialBasisFunction(Basis):
 
         .. seealso:: :func:`OrthogonalPolynomial.GradEvaluate`
         """
-        (out, left, right, diff) = self.Evaluate(x, N, norm, extended_output=True)
+        (out, diff) = self.Evaluate(x, N, norm, extended_output=True)
         if k == 0:
             return out
         elif k == 1:
-            out[left] *= -diff[left]/self.sigma[N-1]**2.
-            out[right] *= -diff[right]/self.sigma[N]**2.
+            out *= -diff/self.sigma[N]**2.
         elif k == 2:
-            out[left] *= ( -1./self.sigma[N-1]**2. + diff[left]**2./self.sigma[N-1]**4. )
-            out[right] *= ( -1./self.sigma[N]**2. + diff[right]**2./self.sigma[N]**4. )
+            out *= ( -1./self.sigma[N]**2. + diff**2./self.sigma[N]**4. )
         elif k == 3:
-            out[left] *= diff[left] * (3.*self.sigma[N-1]**2. - diff[left]**2.) / self.sigma[N-1]**6.
-            out[right] *= diff[right] * (3.*self.sigma[N]**2. - diff[right]**2.) / self.sigma[N]**6.
+            out *= diff * (3.*self.sigma[N]**2. - diff**2.) / self.sigma[N]**6.
         else:
             raise ValueError("%d-th derivative not defined yet" % k)
         return out
@@ -232,6 +230,12 @@ class HermiteProbabilistsRadialBasisFunction(Basis):
             DVr[:,i] = self.GradEvaluate(r, i, k, norm)
         return DVr
 
+    @staticmethod
+    def from_xml_element(node):
+        nbasis = int( node.attrib['nbasis'] )
+        scale = float( node.attrib.get('scale', default=1.) )
+        return HermiteProbabilistsRadialBasisFunction(nbasis, scale=scale)
+
 class ConstantExtendedHermiteProbabilistsRadialBasisFunction(Basis):
     r""" Construction of the Hermite Probabilists' Radial Basis Functions
 
@@ -241,51 +245,36 @@ class ConstantExtendedHermiteProbabilistsRadialBasisFunction(Basis):
     .. math::
 
        \phi_0(x) = 1 \\
-       \phi_i(x) = \begin{cases}
-       \exp\left( -\frac{(x-x_i)^2}{2\sigma^2_{i-1}} \right) & \text{if } x \leq x_i
-       \exp\left( -\frac{(x-x_i)^2}{2\sigma^2_{i}} \right) & \text{if } x > x_i
-       \end{cases}
+       \phi_i(x) = \exp\left( -\frac{(x-x_i)^2}{2\sigma^2_{i-1}} \right) 
 
     where :math:`\sigma_i=x_{i+1} - x_{i}`, :math:`\sigma_0=\sigma_1` and
     :math:`\sigma_N=\sigma_{N-1}`
 
     Args:
-      order (int): maximum order :math:`M`
+      nbasis (int): number of basis :math:`M`
       scale (float): scaling for the badwidth :math:`\sigma`.
     """
 
     def __init__(self, nbasis, scale=1.):
         self.nbasis = nbasis
-        self.nknots = nbasis - 1
-        self.hp = HermiteProbabilistsPolynomial()
-        self.xknots, self.wknots = self.hp.GaussQuadrature(self.nknots-1)
-        self.sigma = np.zeros(self.nknots+1)
-        if self.nknots == 1:
-            self.sigma[:] = 1.
-        else:
-            self.sigma[1:-1] = np.diff(self.xknots)
-            self.sigma[0] = self.sigma[1]
-            self.sigma[-1] = self.sigma[-2]
-        self.sigma *= scale
+        if nbasis >= 1:
+            self.hprbf = HermiteProbabilistsRadialBasisFunction(nbasis-1, scale=scale)
 
     def Quadrature(self, N, quadType=None, norm=False):
         r""" Generate quadrature rules of the selected type.
 
         .. seealso:: :func:`OrthogonalPolynomial.Quadrature`
         """
-        if quadType in [None, GAUSS]:
-            return self.GaussQuadrature(N, norm)
-        else:
-            raise ValueError("quadType=%s not available" % quadType)
+        return self.hprbf.Quadrature(N, quadType, norm)
 
     def GaussQuadrature(self, N, norm=False):
         r""" Hermite Probabilists' function Gauss quadratures
 
         .. seealso:: :func:`OrthogonalPolynomial.GaussQuadrature`
         """
-        return self.hp.GaussQuadrature(N, norm)
+        return self.hprbf.GaussQuadrature(N, norm)
 
-    def Evaluate(self, x, N, norm=True, extended_output=False):
+    def Evaluate(self, x, N, norm=True):
         r""" Evaluate the ``N``-th Hermite Probabilists' Radial Basis Function
 
         .. seealso:: :func:`OrthogonalPolynomial.Evaluate`
@@ -294,48 +283,24 @@ class ConstantExtendedHermiteProbabilistsRadialBasisFunction(Basis):
             raise ValueError("N must be <= than the number of basis")
         if N == 0:
             out = np.ones(x.shape[0])
-            left = None
-            right = None
-            diff = None
         else:
             nrbf = N-1
-            leq = (x <= self.xknots[nrbf])
-            left = np.where(leq)[0]
-            right = np.where(np.logical_not(leq))[0]
-            out = np.zeros(x.shape[0])
-            diff = x - self.xknots[nrbf]
-            out[left] = np.exp( - (diff[left])**2./(2.*self.sigma[nrbf-1]**2.) )
-            out[right] = np.exp( - (diff[right])**2./(2.*self.sigma[nrbf]**2.) )
-        if extended_output:
-            return (out, left, right, diff)
-        else:
-            return out
+            out = self.hprbf.Evaluate(x, nrbf, norm)
+        return out
 
     def GradEvaluate(self, x, N, k=0, norm=True):
         r""" Evaluate the ``k``-th derivative of the ``N``-th Hermite Probabilists' Radial Basis Function
 
         .. seealso:: :func:`OrthogonalPolynomial.GradEvaluate`
         """
-        if N == 0 and k > 0:
-            return np.zeros(x.shape[0])
-        (out, left, right, diff) = self.Evaluate(x, N, norm, extended_output=True)
-        nrbf = N-1
-        if k == 0:
-            return out
-        elif k == 1:
-            out[left] *= -diff[left]/self.sigma[nrbf-1]**2.
-            out[right] *= -diff[right]/self.sigma[nrbf]**2.
-        elif k == 2:
-            out[left] *= ( -1./self.sigma[nrbf-1]**2. + diff[left]**2./self.sigma[nrbf-1]**4. )
-            out[right] *= ( -1./self.sigma[nrbf]**2. + diff[right]**2./self.sigma[nrbf]**4. )
-        elif k == 3:
-            out[left] *= diff[left] * (3.*self.sigma[nrbf-1]**2. - diff[left]**2.) / \
-                         self.sigma[nrbf-1]**6.
-            out[right] *= diff[right] * (3.*self.sigma[nrbf]**2. - diff[right]**2.) / \
-                          self.sigma[nrbf]**6.
+        if N == 0:
+            if k == 0:
+                return np.ones(x.shape[0])
+            elif k > 0:
+                return np.zeros(x.shape[0])
         else:
-            raise ValueError("%d-th derivative not defined yet" % k)
-        return out
+            nrbf = N-1
+            return self.hprbf.GradEvaluate(x, nrbf, k, norm)
 
     def GradVandermonde(self, r, N, k=0, norm=True):
         r""" Generate the ``k``-th derivative of the ``N``-th order Vandermoned matrix.
@@ -358,6 +323,12 @@ class ConstantExtendedHermiteProbabilistsRadialBasisFunction(Basis):
             DVr[:,i] = self.GradEvaluate(r, i, k, norm)
         return DVr
 
+    @staticmethod
+    def from_xml_element(node):
+        nbasis = int( node.attrib['nbasis'] )
+        scale = float( node.attrib.get('scale', default=1.) )
+        return ConstantExtendedHermiteProbabilistsRadialBasisFunction(nbasis, scale=scale)
+
 class LinearExtendedHermiteProbabilistsRadialBasisFunction(Basis):
     r""" Construction of the Hermite Probabilists' Radial Basis Functions
 
@@ -368,49 +339,34 @@ class LinearExtendedHermiteProbabilistsRadialBasisFunction(Basis):
 
        \phi_0(x) = 1 \\
        \phi_1(x) = x \\
-       \phi_i(x) = \begin{cases}
-       \exp\left( -\frac{(x-x_i)^2}{2\sigma^2_{i-1}} \right) & \text{if } x \leq x_i
-       \exp\left( -\frac{(x-x_i)^2}{2\sigma^2_{i}} \right) & \text{if } x > x_i
-       \end{cases}
+       \phi_i(x) = \exp\left( -\frac{(x-x_i)^2}{2\sigma^2_{i-1}} \right) 
 
     where :math:`\sigma_i=x_{i+1} - x_{i}`, :math:`\sigma_0=\sigma_1` and
     :math:`\sigma_N=\sigma_{N-1}`
 
     Args:
-      order (int): maximum order :math:`M`
+      nbasis (int): number of basis :math:`M`
       scale (float): scaling for the badwidth :math:`\sigma`.
     """
 
     def __init__(self, nbasis, scale=1.):
         self.nbasis = nbasis
-        self.nknots = nbasis - 2
-        self.hp = HermiteProbabilistsPolynomial()
-        self.xknots, self.wknots = self.hp.GaussQuadrature(self.nknots-1)
-        self.sigma = np.zeros(self.nknots+1)
-        if self.nknots == 1:
-            self.sigma[:] = 1.
-        else:
-            self.sigma[1:-1] = np.diff(self.xknots)
-            self.sigma[0] = self.sigma[1]
-            self.sigma[-1] = self.sigma[-2]
-        self.sigma *= scale
+        if nbasis >= 2:
+            self.hprbf = HermiteProbabilistsRadialBasisFunction(nbasis-2, scale=scale)
 
     def Quadrature(self, N, quadType=None, norm=False):
         r""" Generate quadrature rules of the selected type.
 
         .. seealso:: :func:`OrthogonalPolynomial.Quadrature`
         """
-        if quadType in [None, GAUSS]:
-            return self.GaussQuadrature(N, norm)
-        else:
-            raise ValueError("quadType=%s not available" % quadType)
+        return self.hprbf.Quadrature(N, quadType, norm)
 
     def GaussQuadrature(self, N, norm=False):
         r""" Hermite Probabilists' function Gauss quadratures
 
         .. seealso:: :func:`OrthogonalPolynomial.GaussQuadrature`
         """
-        return self.hp.GaussQuadrature(N, norm)
+        return self.hprbf.GaussQuadrature(N, norm)
 
     def Evaluate(self, x, N, norm=True, extended_output=False):
         r""" Evaluate the ``N``-th Hermite Probabilists' Radial Basis Function
@@ -421,58 +377,33 @@ class LinearExtendedHermiteProbabilistsRadialBasisFunction(Basis):
             raise ValueError("N must be <= than the number of basis")
         if N == 0:
             out = np.ones(x.shape[0])
-            left = None
-            right = None
-            diff = None
         elif N == 1:
             out = x.flatten()
-            left = None
-            right = None
-            diff = None
         else:
             nrbf = N-2
-            leq = (x <= self.xknots[nrbf])
-            left = np.where(leq)[0]
-            right = np.where(np.logical_not(leq))[0]
-            out = np.zeros(x.shape[0])
-            diff = x - self.xknots[nrbf]
-            out[left] = np.exp( - (diff[left])**2./(2.*self.sigma[nrbf-1]**2.) )
-            out[right] = np.exp( - (diff[right])**2./(2.*self.sigma[nrbf]**2.) )
-        if extended_output:
-            return (out, left, right, diff)
-        else:
-            return out
+            out = self.hprbf.Evaluate(x, nrbf, norm)
+        return out
 
     def GradEvaluate(self, x, N, k=0, norm=True):
         r""" Evaluate the ``k``-th derivative of the ``N``-th Hermite Probabilists' Radial Basis Function
 
         .. seealso:: :func:`OrthogonalPolynomial.GradEvaluate`
         """
-        if N == 0 and k > 0:
-            return np.zeros(x.shape[0])
-        if N == 1:
-            if k == 1:
+        if N == 0:
+            if k == 0:
+                return np.ones(x.shape[0])
+            elif k > 0:
+                return np.zeros(x.shape[0])
+        elif N == 1:
+            if k == 0:
+                return x.flatten()
+            elif k == 1:
                 return np.ones(x.shape[0])
             elif k > 1:
                 return np.zeros(x.shape[0])
-        (out, left, right, diff) = self.Evaluate(x, N, norm, extended_output=True)
-        nrbf = N-2
-        if k == 0:
-            return out
-        elif k == 1:
-            out[left] *= -diff[left]/self.sigma[nrbf-1]**2.
-            out[right] *= -diff[right]/self.sigma[nrbf]**2.
-        elif k == 2:
-            out[left] *= ( -1./self.sigma[nrbf-1]**2. + diff[left]**2./self.sigma[nrbf-1]**4. )
-            out[right] *= ( -1./self.sigma[nrbf]**2. + diff[right]**2./self.sigma[nrbf]**4. )
-        elif k == 3:
-            out[left] *= diff[left] * (3.*self.sigma[nrbf-1]**2. - diff[left]**2.) / \
-                         self.sigma[nrbf-1]**6.
-            out[right] *= diff[right] * (3.*self.sigma[nrbf]**2. - diff[right]**2.) / \
-                          self.sigma[nrbf]**6.
         else:
-            raise ValueError("%d-th derivative not defined yet" % k)
-        return out
+            nrbf = N-2
+            return self.hprbf.GradEvaluate(x, nrbf, k, norm)
 
     def GradVandermonde(self, r, N, k=0, norm=True):
         r""" Generate the ``k``-th derivative of the ``N``-th order Vandermoned matrix.
@@ -493,4 +424,10 @@ class LinearExtendedHermiteProbabilistsRadialBasisFunction(Basis):
         DVr = np.zeros((r.shape[0],N+1))
         for i in range(0,N+1):
             DVr[:,i] = self.GradEvaluate(r, i, k, norm)
-        return DVr
\ No newline at end of file
+        return DVr
+
+    @staticmethod
+    def from_xml_element(node):
+        nbasis = int( node.attrib['nbasis'] )
+        scale = float( node.attrib.get('scale', default=1.) )
+        return LinearExtendedHermiteProbabilistsRadialBasisFunction(nbasis, scale=scale)
\ No newline at end of file
