@@ -525,6 +525,12 @@ class HermiteProbabilistsPolynomial(OrthogonalPolynomial):
         """
         return math.sqrt(2.*np.pi) * factorial(N,exact=True)
 
+    @staticmethod
+    def from_xml_element(node):
+        normalized = node.attrib.get('normalized', default=None)
+        if normalized is not None: normalized = bool(normalized)
+        return HermiteProbabilistsPolynomial(normalized=normalized)
+
 class GenericOrthogonalPolynomial(OrthogonalPolynomial):
     r""" Construction of polynomials orthogonal with respect to a generic measure
 
