@@ -17,37 +17,25 @@
 # along with SpectralToolbox.  If not, see <http://www.gnu.org/licenses/>.
 #
 # DTU UQ Library
-# Copyright (C) 2014 The Technical University of Denmark
+# Copyright (C) 2012-2015 The Technical University of Denmark
 # Scientific Computing Section
 # Department of Applied Mathematics and Computer Science
 #
+# Copyright (C) 2015-2016 Massachusetts Institute of Technology
+# Uncertainty Quantification group
+# Department of Aeronautics and Astronautics
+#
 # Author: Daniele Bigoni
 #
 
-"""
-=================================
- Spectral ND
-=================================
-
-Created on Mon Jul  9 11:35:12 2012
-
-@author: Daniele Bigoni (dabi@imm.dtu.dk)
-
-Implementation of Spectral Methods in n dimension.
-
-It uses the package ``Spectral1D`` as basic polynomials in order to construct higher dimensional rules by tensor product.
-
-"""
-
-import numpy as np
-
-from scipy.misc import comb
-
 import sys
-
+import logging
 import itertools
 
-import Spectral1D
+import numpy as np
+from scipy.misc import comb
+
+import SpectralToolbox.Spectral1D
 
 def MultiIndex(d,N):
     """
@@ -103,8 +91,14 @@ def __MultiIndexRec(IDX,d,m,i):
     return mNew
 
 class PolyND:
-    polys = []
-    DIM = 0
+    
+    logger = logging.getLogger(__name__)
+    logger.propagate = False
+    ch = logging.StreamHandler(sys.stdout)
+    formatter = logging.Formatter("%(asctime)s %(levelname)s:%(name)s: %(message)s",
+                                  "%Y-%m-%d %H:%M:%S")
+    ch.setFormatter(formatter)
+    logger.addHandler(ch)
     
     def __init__(self, polys):
         """
@@ -122,20 +116,17 @@ class PolyND:
         self.polys = polys
         self.DIM = len(self.polys)
 
-    def Quadrature(self, Ns, quadTypes=None,left=None,right=None,end=None, normed=False, warnings=True):
+    def Quadrature(self, Ns, quadTypes=None, norm=True, warnings=True):
         """
         GaussQuadrature(): computes the tensor product of the Guass Points and weights
         
         Syntax:
-            ``(x,w) = GaussQuadrature(Ns, [quadTypes=None], [normed=False],[warnings=True])``
+            ``(x,w) = GaussQuadrature(Ns, [quadTypes=None], [norm=True],[warnings=True])``
         
         Input:
             * ``Ns`` = (list,int) n-dimensional list with the order of approximation of each polynomial
             * ``quadTypes`` = (list,``Spectral1D.AVAIL_QUADPOINTS``) n-dimensional list of quadrature point types chosen among Gauss, Gauss-Radau, Gauss-Lobatto (using the definition in ``Spectral1D``). If None, Gauss points will be generated by default
-            * ``left``: (list,float) list of left values used by ORTHPOL for Gauss-Lobatto rules (the dimensions where the value is not used can be set to anything)
-            * ``right``: (list,float) list of left values used by ORTHPOL for Gauss-Lobatto rules (the dimensions where the value is not used can be set to anything)
-            * ``end``: (list,float) list of left values used by ORTHPOL for Gauss-Radau rules (the dimensions where the value is not used can be set to anything)
-            * ``normed`` = (optional,boolean) whether the weights will be normalized or not
+            * ``norm`` = (optional,boolean) whether the weights will be normalized or not
             * ``warnings`` = (optional,boolean) set whether to ask for confirmation when it is required to allocate more then 100Mb of memory
         
         Output:
@@ -150,24 +141,14 @@ class PolyND:
         warningMem = 100.0
         
         if self.DIM != len(Ns) :
-            print "The number of elements in Ns is not consistent"
+            print("The number of elements in Ns is not consistent")
             return
 
         if quadTypes == None:
             quadTypes = [Spectral1D.GAUSS for i in range(self.DIM)]
-        if left == None:
-            left = [None for i in range(self.DIM)]
-        if right == None:
-            right = [None for i in range(self.DIM)]
-        if end == None:
-            end = [None for i in range(self.DIM)]
-
-        if self.DIM != len(quadTypes) or self.DIM != len(left) or self.DIM != len(right) or self.DIM != len(end):
-            print "The number of elements in quadTypes, left, right, end are not consistent"
-            return
 
         if self.DIM != len(quadTypes):
-            print "The number of elements in quadTypes is not consistent"
+            print("The number of elements in quadTypes is not consistent")
             return
         
         #######################
@@ -179,46 +160,45 @@ class PolyND:
         wMem = Ncoll * SDOUBLE + SARRAY
         totMem = xMem + wMem
         # Print out information
-        print( "----------------------------------------------------------------------" )
-        print( "SpectralND.GaussQuadrature: Memory usage information:" )
-        print( "\t X Points: %10.2f Mb " % (xMem * 1e-6) )
-        print( "\t Weights: %10.2f Mb " % (wMem * 1e-6) )
-        print( "Total Memory: %10.2f Mb " % (totMem * 1e-6) )
-        print( "N of collocation points: %d " % (Ncoll) )
+        self.logger.info("\n" +
+                         "Memory usage information:\n" +
+                         "\t X Points: %10.2f Mb \n" % (xMem * 1e-6) +
+                         "\t Weights: %10.2f Mb \n" % (wMem * 1e-6) +
+                         "Total Memory: %10.2f Mb \n" % (totMem * 1e-6) +
+                         "N of collocation points: %d " % (Ncoll))
         
         if warnings and totMem * 1e-6 > warningMem:
             opt = 'a'
             while (opt != 'c' and opt != 'b' and opt != 'q'):
-                print("The total memory that will be allocated exceed %10.2fMb. Chose one , of the following options:" % (warningMem))
-                print("\t [c]: continue")
-                print("\t [q]: exit")
+                self.logger.warning("\n" +
+                                    "The total memory that will be allocated exceed %10.2fMb. Chose one , of the following options:\n" % (warningMem) +
+                                    "\t [c]: continue\n" +
+                                    "\t [q]: exit" )
                 opt = sys.stdin.read(1)
             if (opt ==  'q'):
-                return
+                return        
         
-        print( "----------------------------------------------------------------------" )
-        
-        x,w = self.polys[0].Quadrature(Ns[0],quadType=quadTypes[0],left=left[0],right=right[0],end=end[0],normed=normed)
+        x,w = self.polys[0].Quadrature(Ns[0],quadType=quadTypes[0],norm=norm)
         wKron = w
         xs = [x]
         for i in range(1,self.DIM):
-            x,w = self.polys[i].Quadrature(Ns[i],quadType=quadTypes[i],left=left[i],right=right[i],end=end[i],normed=normed)
+            x,w = self.polys[i].Quadrature(Ns[i],quadType=quadTypes[i],norm=norm)
             wKron = np.kron(wKron, w)
             xs.append(x)
         xKron = np.asarray(list(itertools.product(*xs)))
         
         return (xKron, wKron)
     
-    def GaussQuadrature(self, Ns, normed=False, warnings=True):
+    def GaussQuadrature(self, Ns, norm=True, warnings=True):
         """
         GaussQuadrature(): computes the tensor product of the Guass Points and weights
         
         Syntax:
-            ``(x,w) = GaussQuadrature(Ns, [normed=False],[warnings=True])``
+            ``(x,w) = GaussQuadrature(Ns, [norm=True],[warnings=True])``
         
         Input:
             * ``Ns`` = (list,int) n-dimensional list with the order of approximation of each polynomial
-            * ``normed`` = (optional,boolean) whether the weights will be normalized or not
+            * ``norm`` = (optional,boolean) whether the weights will be normalized or not
             * ``warnings`` = (optional,boolean) set whether to ask for confirmation when it is required to allocate more then 100Mb of memory
         
         Output:
@@ -233,7 +213,7 @@ class PolyND:
         warningMem = 100.0
         
         if self.DIM != len(Ns) :
-            print "The number of elements in Ns is not consistent"
+            print("The number of elements in Ns is not consistent")
             return
 
         #######################
@@ -245,46 +225,45 @@ class PolyND:
         wMem = Ncoll * SDOUBLE + SARRAY
         totMem = xMem + wMem
         # Print out information
-        print( "----------------------------------------------------------------------" )
-        print( "SpectralND.GaussQuadrature: Memory usage information:" )
-        print( "\t X Points: %10.2f Mb " % (xMem * 1e-6) )
-        print( "\t Weights: %10.2f Mb " % (wMem * 1e-6) )
-        print( "Total Memory: %10.2f Mb " % (totMem * 1e-6) )
-        print( "N of collocation points: %d " % (Ncoll) )
+        self.logger.info("\n" +
+                         "Memory usage information:\n" +
+                         "\t X Points: %10.2f Mb \n" % (xMem * 1e-6) +
+                         "\t Weights: %10.2f Mb \n" % (wMem * 1e-6) +
+                         "Total Memory: %10.2f Mb \n" % (totMem * 1e-6) +
+                         "N of collocation points: %d " % (Ncoll))
         
         if warnings and totMem * 1e-6 > warningMem:
             opt = 'a'
             while (opt != 'c' and opt != 'b' and opt != 'q'):
-                print("The total memory that will be allocated exceed %10.2fMb. Chose one , of the following options:" % (warningMem))
-                print("\t [c]: continue")
-                print("\t [q]: exit")
+                self.logger.warning("\n" +
+                                    "The total memory that will be allocated exceed %10.2fMb. Chose one , of the following options:\n" % (warningMem) +
+                                    "\t [c]: continue\n" +
+                                    "\t [q]: exit" )
                 opt = sys.stdin.read(1)
             if (opt ==  'q'):
                 return
-        
-        print( "----------------------------------------------------------------------" )
-        
-        x,w = self.polys[0].GaussQuadrature(Ns[0],normed=normed)
+                
+        x,w = self.polys[0].GaussQuadrature(Ns[0],norm=norm)
         wKron = w
         xs = [x]
         for i in range(1,self.DIM):
-            x,w = self.polys[i].GaussQuadrature(Ns[i],normed=normed)
+            x,w = self.polys[i].GaussQuadrature(Ns[i],norm=norm)
             wKron = np.kron(wKron, w)
             xs.append(x)
         xKron = np.asarray(list(itertools.product(*xs)))
         
         return (xKron, wKron)
     
-    def GaussLobattoQuadrature(self, Ns, normed=False, warnings=True):
+    def GaussLobattoQuadrature(self, Ns, norm=True, warnings=True):
         """
         GaussLobattoQuadrature(): computes the tensor product of the Guass Lobatto Points and weights
         
         Syntax:
-            ``(x,w) = GaussLobattoQuadrature(Ns,[normed=False],[warnings=True])``
+            ``(x,w) = GaussLobattoQuadrature(Ns,[norm=True],[warnings=True])``
         
         Input:
             * ``Ns`` = (list,int) n-dimensional list with the order of approximation of each polynomial
-            * ``normed`` = (optional,boolean) whether the weights will be normalized or not
+            * ``norm`` = (optional,boolean) whether the weights will be normalized or not
             * ``warnings`` = (optional,boolean) set whether to ask for confirmation when it is required to allocate more then 100Mb of memory
         
         Output:
@@ -299,7 +278,7 @@ class PolyND:
         warningMem = 100.0
         
         if self.DIM != len(Ns) :
-            print "The number of elements in Ns is not consistent"
+            print("The number of elements in Ns is not consistent")
             return
         
         #######################
@@ -311,25 +290,24 @@ class PolyND:
         wMem = Ncoll * SDOUBLE + SARRAY
         totMem = xMem + wMem
         # Print out information
-        print( "----------------------------------------------------------------------" )
-        print( "SpectralND.GaussLobattoQuadrature: Memory usage information:" )
-        print( "\t X Points: %10.2f Mb " % (xMem * 1e-6) )
-        print( "\t Weights: %10.2f Mb " % (wMem * 1e-6) )
-        print( "Total Memory: %10.2f Mb " % (totMem * 1e-6) )
-        print( "N of collocation points: %d " % (Ncoll) )
+        self.logger.info("\n" +
+                         "Memory usage information:\n" +
+                         "\t X Points: %10.2f Mb \n" % (xMem * 1e-6) +
+                         "\t Weights: %10.2f Mb \n" % (wMem * 1e-6) +
+                         "Total Memory: %10.2f Mb \n" % (totMem * 1e-6) +
+                         "N of collocation points: %d " % (Ncoll))
         
         if warnings and totMem * 1e-6 > warningMem:
             opt = 'a'
             while (opt != 'c' and opt != 'b' and opt != 'q'):
-                print("The total memory that will be allocated exceed %10.2fMb. Chose one , of the following options:" % (warningMem))
-                print("\t [c]: continue")
-                print("\t [q]: exit")
+                self.logger.warning("\n" +
+                                    "The total memory that will be allocated exceed %10.2fMb. Chose one , of the following options:\n" % (warningMem) +
+                                    "\t [c]: continue\n" +
+                                    "\t [q]: exit" )
                 opt = sys.stdin.read(1)
             if (opt ==  'q'):
                 return
-        
-        print( "----------------------------------------------------------------------" )
-        
+
         x,w = self.polys[0].GaussLobattoQuadrature(Ns[0])
         wKron = w
         xs = [x]
@@ -341,7 +319,7 @@ class PolyND:
         
         return (xKron, wKron)
     
-    def GradVandermonde(self,rs,Ns,ks,norms=None,usekron=True,output=True,warnings=True):
+    def GradVandermonde(self,rs,Ns,ks=None,norms=None,usekron=True,output=True,warnings=True):
         """
         GradVandermonde(): initialize the tensor product of the k-th gradient of the modal basis.
         
@@ -351,7 +329,7 @@ class PolyND:
         Input:
             * ``rs`` = (list of 1d-array,float) ``n``-dimensional list of set of points on which to evaluate the polynomials (by default they are not the kron product of the points. See ``usekron`` option)
             * ``Ns`` = (list,int) n-dimensional list with the maximum orders of approximation of each polynomial
-            * ``ks`` = (list,int) n-dimensional list with derivative orders
+            * ``ks`` = (list,int) n-dimensional list with derivative orders [default=0]
             * ``norms`` = (default=None,list,boolean) n-dimensional list of boolean, True -> orthonormal, False -> orthogonal, None -> all orthonormal
             * ``usekron`` = (optional,boolean) set whether to apply the kron product of the single dimensional Vandermonde matrices or to multiply column-wise. kron(rs) and usekron==False is equal to rs and usekron==True
             * ``output`` = (optional,boolean) set whether to print out information about memory allocation
@@ -366,19 +344,21 @@ class PolyND:
         
         # Memory allocation for which the user will get a warning message (Mb)
         warningMem = 100.0
+
+        if norms == None:
+            norms = [True] * self.DIM
+        if ks == None:
+            ks = [0]*self.DIM
         
         if usekron:
-            if (self.DIM != len(rs) or self.DIM != len(Ns) or self.DIM != len(ks) or ( norms != None and self.DIM != len(norms))) :
-                print "The number of elements in rs, Ns, ks and norms is not consistent"
+            if (self.DIM != len(rs) or self.DIM != len(Ns) or self.DIM != len(ks) or self.DIM != len(norms)) :
+                print("The number of elements in rs, Ns, ks and norms is not consistent")
                 return
         else:
-            if ( not (type(rs) == np.ndarray and rs.shape[1] == self.DIM) or self.DIM != len(Ns) or self.DIM != len(ks) or ( norms != None and self.DIM != len(norms))) :
-                print "The number of elements in rs, Ns, ks and norms is not consistent"
+            if ( not (type(rs) == np.ndarray and rs.shape[1] == self.DIM) or self.DIM != len(Ns) or self.DIM != len(ks) or self.DIM != len(norms)) :
+                print("The number of elements in rs, Ns, ks and norms is not consistent")
                 return
-        
-        if norms == None:
-            norms = np.ones(self.DIM)
-        
+                
         #######################
         # Estimate memory usage
         if usekron:
@@ -396,40 +376,38 @@ class PolyND:
             VMem = np.prod(np.asarray(Ns)+1) * Ncolls * SDOUBLE + SARRAY
             totMem = VMem
         # Print out information
-        if output:
-            print( "----------------------------------------------------------------------" )
-            print( "SpectralND.GradVandermonde: Memory usage information:" )
-            print( "\t Tensor Basis: %10.2f Mb " % (VMem * 1e-6) )
-            print( "Total Memory: %10.2f Mb " % (totMem * 1e-6) )
-            
-            if warnings and totMem * 1e-6 > warningMem:
-                opt = 'a'
-                while (opt != 'c' and opt != 'b' and opt != 'q'):
-                    print("The total memory that will be allocated exceed %10.2fMb. Chose one , of the following options:" % (warningMem))
-                    print("\t [c]: continue")
-                    print("\t [q]: exit")
-                    opt = sys.stdin.read(1)
-                if (opt ==  'q'):
-                    return
-            
-            print( "----------------------------------------------------------------------" )
-        
+        self.logger.info("\n" +
+                         "Memory usage information:\n" +
+                         "\t Tensor Basis: %10.2f Mb \n" % (VMem * 1e-6) +
+                         "Total Memory: %10.2f Mb " % (totMem * 1e-6) )
+
+        if output and warnings and totMem * 1e-6 > warningMem:
+            opt = 'a'
+            while (opt != 'c' and opt != 'b' and opt != 'q'):
+                self.logger.warning("%n" +
+                                    "The total memory that will be allocated exceed %10.2fMb. Chose one , of the following options:\n" % (warningMem) +
+                                    "\t [c]: continue\n" +
+                                    "\t [q]: exit")
+                opt = sys.stdin.read(1)
+            if (opt ==  'q'):
+                return
+
         if usekron:
-            VKron = self.polys[0].GradVandermonde1D(rs[0],Ns[0],ks[0],norms[0])
+            VKron = self.polys[0].GradVandermonde(rs[0],Ns[0],ks[0],norms[0])
             for i in range(1,self.DIM):
-                VKron = np.kron(VKron, self.polys[i].GradVandermonde1D(rs[i],Ns[i],ks[i],norms[i]))
+                VKron = np.kron(VKron, self.polys[i].GradVandermonde(rs[i],Ns[i],ks[i],norms[i]))
         else:
             VKron = np.ones((rs.shape[0],1))
             for i in range(0,self.DIM):
                 VKronNew = np.zeros((VKron.shape[0],VKron.shape[1] * (Ns[i]+1)))
-                V = self.polys[i].GradVandermonde1D(rs[:,i],Ns[i],ks[i],norms[i])
+                V = self.polys[i].GradVandermonde(rs[:,i],Ns[i],ks[i],norms[i])
                 for col in range(0,VKron.shape[1]):
                     VKronNew[:,col*(Ns[i]+1):(col+1)*(Ns[i]+1)] = np.tile(VKron[:,col],(Ns[i]+1,1)).T * V
                 VKron = VKronNew
         
         return VKron
 
-    def GradVandermondePascalSimplex(self,rs,N,ks,norms=None,usekron=True,output=True,warnings=True):
+    def GradVandermondePascalSimplex(self,rs,N,ks=None,norms=None,usekron=True,output=True,warnings=True):
         """
         GradVandermondePascalSimplex(): initialize k-th gradient of the modal basis up to the total order N
         
@@ -439,7 +417,7 @@ class PolyND:
         Input:
             * ``rs`` = (list of 1d-array,float) ``n``-dimensional list of set of points on which to evaluate the polynomials (by default they are not the kron product of the points. See ``usekron`` option)
             * ``N`` = (int) the maximum orders of the polynomial basis
-            * ``ks`` = (list,int) n-dimensional list with derivative orders
+            * ``ks`` = (list,int) n-dimensional list with derivative orders [default=0]
             * ``norms`` = (default=None,list,boolean) n-dimensional list of boolean, True -> orthonormal, False -> orthogonal, None -> all orthonormal
             * ``usekron`` = (optional,boolean) set whether to apply the kron product of the single dimensional Vandermonde matrices or to multiply column-wise. kron(rs) and usekron==False is equal to rs and usekron==True
             * ``output`` = (optional,boolean) set whether to print out information about memory allocation
@@ -454,19 +432,21 @@ class PolyND:
         
         # Memory allocation for which the user will get a warning message (Mb)
         warningMem = 100.0
+
+        if norms == None:
+            norms = [True] * self.DIM
+        if ks == None:
+            ks = [0]*self.DIM
         
         if usekron:
-            if (self.DIM != len(rs) or self.DIM != len(ks) or ( norms != None and self.DIM != len(norms))) :
-                print "The number of elements in rs, ks and norms is not consistent"
+            if (self.DIM != len(rs) or self.DIM != len(ks) or self.DIM != len(norms)) :
+                print("The number of elements in rs, ks and norms is not consistent")
                 return
         else:
-            if ( not (type(rs) == np.ndarray and rs.shape[1] == self.DIM) or self.DIM != len(ks) or ( norms != None and self.DIM != len(norms))) :
-                print "The number of elements in rs, ks and norms is not consistent"
+            if ( not (type(rs) == np.ndarray and rs.shape[1] == self.DIM) or self.DIM != len(ks) or self.DIM != len(norms)) :
+                print("The number of elements in rs, ks and norms is not consistent")
                 return
         
-        if norms == None:
-            norms = np.ones(self.DIM)
-        
         #######################
         # Estimate memory usage
         if usekron:
@@ -492,23 +472,21 @@ class PolyND:
             VMem = Nbasis * Ncolls * SDOUBLE + SARRAY
             totMem = VMem
         # Print out information
-        if output:
-            print( "----------------------------------------------------------------------" )
-            print( "SpectralND.GradVandermondePascalSimplex Memory usage information:" )
-            print( "\t Tensor Basis: %10.2f Mb " % (VMem * 1e-6) )
-            print( "Total Memory: %10.2f Mb " % (totMem * 1e-6) )
+        self.logger.info("\n" +
+                         "Memory usage information:\n" +
+                         "\t Tensor Basis: %10.2f Mb \n" % (VMem * 1e-6) +
+                         "Total Memory: %10.2f Mb " % (totMem * 1e-6) )
             
-            if warnings and totMem * 1e-6 > warningMem:
-                opt = 'a'
-                while (opt != 'c' and opt != 'b' and opt != 'q'):
-                    print("The total memory that will be allocated exceed %10.2fMb. Chose one , of the following options:" % (warningMem))
-                    print("\t [c]: continue")
-                    print("\t [q]: exit")
-                    opt = sys.stdin.read(1)
-                if (opt ==  'q'):
-                    return
-            
-            print( "----------------------------------------------------------------------" )
+        if output and warnings and totMem * 1e-6 > warningMem:
+            opt = 'a'
+            while (opt != 'c' and opt != 'b' and opt != 'q'):
+                self.logger.warning("%n" +
+                                    "The total memory that will be allocated exceed %10.2fMb. Chose one , of the following options:\n" % (warningMem) +
+                                    "\t [c]: continue\n" +
+                                    "\t [q]: exit")
+                opt = sys.stdin.read(1)
+            if (opt ==  'q'):
+                return
         
         if usekron:
             # Compute combinations of collocation points
@@ -519,7 +497,7 @@ class PolyND:
         # Compute single Generalized Vandermonde Matrices
         Vs = []
         for i in range(0,self.DIM):
-            Vs.append(self.polys[i].GradVandermonde1D(xKron[:,i],N,ks[i],norms[i]))
+            Vs.append(self.polys[i].GradVandermonde(xKron[:,i],N,ks[i],norms[i]))
         
         # Make space for the Vandermonde matrix
         V = np.ones((Ncolls,Nbasis))
