@@ -1,7 +1,5 @@
 # -*- coding: utf-8 -*-
 
-#!/usr/bin/env python
-
 #
 # This file is part of SpectralToolbox.
 #
@@ -19,19 +17,26 @@
 # along with SpectralToolbox.  If not, see <http://www.gnu.org/licenses/>.
 #
 # DTU UQ Library
-# Copyright (C) 2014 The Technical University of Denmark
+# Copyright (C) 2012-2015 The Technical University of Denmark
 # Scientific Computing Section
 # Department of Applied Mathematics and Computer Science
 #
+# Copyright (C) 2015-2016 Massachusetts Institute of Technology
+# Uncertainty Quantification group
+# Department of Aeronautics and Astronautics
+#
 # Author: Daniele Bigoni
 #
 
-from _version import __version__
+from SpectralToolbox._version import __version__
+
+import SpectralToolbox.tests
 
 __all__ = ["SparseGrids", "Spectral1D", "SpectralND", "Stroud", "HeterogeneousSparseGrids", "Misc"]
+__all__ += ['tests']
 
 __author__ = "Daniele Bigoni"
-__copyright__ = """Copyright 2012-2014, The Technical University of Denmark"""
+__copyright__ = """LGPLv3, Copyright (C) 2012-2015, The Technical University of Denmark"""
 __credits__ = ["Daniele Bigoni"]
 __maintainer__ = "Daniele Bigoni"
 __email__ = "dabi@imm.dtu.dk"
